@@ -16,7 +16,6 @@ class Borg:
     """Definition of a class that can never be duplicated. Correct usage is
     thus:
         
-    >>> from borg import Borg
     >>> class foo(Borg):
     ...     # All attributes on a borg class *must* = None
     ...     attribute = None
@@ -43,11 +42,10 @@ class Borg:
         if borgtype is None:
             raise TypeError('Borg::__init__: You must pass a borgtype')
         if borgtype not in self.__shared_state:
-            dbg('Borg::__init__: Preparing borg state for %s' % borgtype)
+            dbg('Preparing borg state for %s' % borgtype)
             self.__shared_state[borgtype] = {}
         self.__dict__ = self.__shared_state[borgtype]
 
     def prepare_attributes(self):
         """This should be used to prepare any attributes of the borg class."""
         raise NotImplementedError('prepare_attributes')
-
