@@ -6,8 +6,10 @@ import copy
 import os
 import gi
 gi.require_version('Vte', '2.91')
-from gi.repository import Gtk, Gdk, Vte, GdkX11
+from gi.repository import Gtk, Gdk, Vte
 from gi.repository.GLib import GError
+import itertools
+import random
 
 from . import borg
 from .borg import Borg
@@ -15,8 +17,13 @@ from .config import Config
 from .keybindings import Keybindings
 from .util import dbg, err, enumerate_descendants
 from .factory import Factory
-from .cwd import get_pid_cwd
-from .version import APP_NAME, APP_VERSION
+from .translation import _
+
+try:
+    from gi.repository import GdkX11
+except ImportError:
+    dbg("could not import X11 gir module")
+
 
 def eventkey2gdkevent(eventkey):  # FIXME FOR GTK3: is there a simpler way of casting from specific EventKey to generic (union) GdkEvent?
     gdkevent = Gdk.Event.new(eventkey.type)
@@ -48,10 +55,7 @@ class Terminator(Borg):
     origcwd = None
     dbus_path = None
     dbus_name = None
-    pid_cwd = None
-    gnome_client = None
     debug_address = None
-    ibus_running = None
 
     doing_layout = None
     layoutname = None
@@ -92,10 +96,6 @@ class Terminator(Borg):
             self.style_providers = []
         if not self.doing_layout:
             self.doing_layout = False
-        if not self.pid_cwd:
-            self.pid_cwd = get_pid_cwd()
-        if self.gnome_client is None:
-            self.attempt_gnome_client()
         self.connect_signals()
 
     def connect_signals(self):
@@ -117,30 +117,6 @@ class Terminator(Borg):
             self.dbus_name = dbus_service.bus_name.get_name()
             self.dbus_path = dbus_service.bus_path
 
-    def attempt_gnome_client(self):
-        """Attempt to find a GNOME Session to register with"""
-        try:
-            from gi.repository import Gnome
-            self.gnome_program = Gnome.init(APP_NAME, APP_VERSION)  # VERIFY FOR GTK3
-            self.gnome_client = Gnome.Ui.master_client()  # VERIFY FOR GTK3
-            self.gnome_client.connect_to_session_manager()
-            self.gnome_client.connect('save-yourself', self.save_yourself)
-            self.gnome_client.connect('die', self.die)
-            dbg('GNOME session support enabled and registered')
-        except (ImportError, AttributeError):
-            self.gnome_client = False
-            dbg('GNOME session support not available')
-
-    def save_yourself(self, *args):
-        """Save as much state as possible for the session manager"""
-        dbg('preparing session manager state')
-        # FIXME: Implement this
-
-    def die(self, *args):
-        """Die at the hands of the session manager"""
-        dbg('session manager asked us to die')
-        # FIXME: Implement this
-
     def get_windows(self):
         """Return a list of windows"""
         return self.windows
@@ -148,14 +124,12 @@ class Terminator(Borg):
     def register_window(self, window):
         """Register a new window widget"""
         if window not in self.windows:
-            dbg('Terminator::register_window: registering %s:%s' % (id(window),
-                type(window)))
+            dbg('registering %s:%s' % (id(window), type(window)))
             self.windows.append(window)
 
     def deregister_window(self, window):
         """de-register a window widget"""
-        dbg('Terminator::deregister_window: de-registering %s:%s' %
-                (id(window), type(window)))
+        dbg('de-registering %s:%s' % (id(window), type(window)))
         if window in self.windows:
             self.windows.remove(window)
         else:
@@ -169,14 +143,12 @@ class Terminator(Borg):
     def register_launcher_window(self, window):
         """Register a new launcher window widget"""
         if window not in self.launcher_windows:
-            dbg('Terminator::register_launcher_window: registering %s:%s' % (id(window),
-                type(window)))
+            dbg('registering %s:%s' % (id(window), type(window)))
             self.launcher_windows.append(window)
 
     def deregister_launcher_window(self, window):
         """de-register a launcher window widget"""
-        dbg('Terminator::deregister_launcher_window: de-registering %s:%s' %
-                (id(window), type(window)))
+        dbg('de-registering %s:%s' % (id(window), type(window)))
         if window in self.launcher_windows:
             self.launcher_windows.remove(window)
         else:
@@ -190,13 +162,13 @@ class Terminator(Borg):
     def register_terminal(self, terminal):
         """Register a new terminal widget"""
         if terminal not in self.terminals:
-            dbg('Terminator::register_terminal: registering %s:%s' %
+            dbg('registering %s:%s' %
                     (id(terminal), type(terminal)))
             self.terminals.append(terminal)
 
     def deregister_terminal(self, terminal):
         """De-register a terminal widget"""
-        dbg('Terminator::deregister_terminal: de-registering %s:%s' %
+        dbg('de-registering %s:%s' %
                 (id(terminal), type(terminal)))
         self.terminals.remove(terminal)
 
@@ -205,8 +177,7 @@ class Terminator(Borg):
             for window in self.windows:
                 window.destroy()
         else:
-            dbg('Terminator::deregister_terminal: %d terminals remain' %
-                    len(self.terminals))
+            dbg('%d terminals remain' % len(self.terminals))
 
     def find_terminal_by_uuid(self, uuid):
         """Search our terminals for one matching the supplied UUID"""
@@ -238,6 +209,7 @@ class Terminator(Borg):
         window.add(terminal)
         window.show(True)
         terminal.spawn_child()
+        terminal.emit('tab-change', 0)
 
         return(window, terminal)
 
@@ -340,6 +312,7 @@ class Terminator(Borg):
         """Layout operations have finished, record that fact"""
         self.doing_layout = False
         maker = Factory()
+        t = 0
 
         window_last_active_term_mapping = {}
         for window in self.windows:
@@ -354,39 +327,12 @@ class Terminator(Borg):
                 terminal.spawn_child()
 
         for window in self.windows:
-            if window.is_child_notebook():
-                # For windows with a notebook
-                notebook = window.get_toplevel().get_children()[0]
-                # Cycle through pages by number
-                for page in range(0, notebook.get_n_pages()):
-                    # Try and get the entry in the previously saved mapping
-                    mapping = window_last_active_term_mapping[window]
-                    page_last_active_term = mapping.get(notebook.get_nth_page(page),  None)
-                    if page_last_active_term is None:
-                        # Couldn't find entry, so we find the first child of type Terminal
-                        children = notebook.get_nth_page(page).get_children()
-                        for page_last_active_term in children:
-                            if maker.isinstance(page_last_active_term, 'Terminal'):
-                                page_last_active_term = page_last_active_term.uuid
-                                break
-                        else:
-                            err('Should never reach here!')
-                            page_last_active_term = None
-                    if page_last_active_term is None:
-                        # Bail on this tab as we're having no luck here, continue with the next
-                        continue
-                    # Set the notebook entry, then ensure Terminal is visible and focussed
-                    urn = page_last_active_term.urn
-                    notebook.last_active_term[notebook.get_nth_page(page)] = page_last_active_term
-                    if urn:
-                        term = self.find_terminal_by_uuid(urn)
-                        if term:
-                            term.ensure_visible_and_focussed()
-            else:
-                # For windows without a notebook ensure Terminal is visible and focussed
+            if not window.is_child_notebook():
+                # For windows without a notebook ensure Terminal is visible and focused
                 if window_last_active_term_mapping[window]:
                     term = self.find_terminal_by_uuid(window_last_active_term_mapping[window].urn)
-                    term.ensure_visible_and_focussed()
+                    if term:
+                        term.ensure_visible_and_focussed()
 
         # Build list of new windows using prelayout list
         new_win_list = []
@@ -401,26 +347,15 @@ class Terminator(Borg):
             window.grab_focus()
             try:
                 t = GdkX11.x11_get_server_time(window.get_window())
-            except (TypeError, AttributeError):
+            except (NameError,TypeError, AttributeError):
                 t = 0
             window.get_window().focus(t)
 
-        # Awful workaround to be sure that the last focused window is actually the one focused.
-        # Don't ask, don't tell policy on this. Even this is not 100%
+        # Going by the docs, this should be all that's needed to ensure that the
+        # last_active_window is focussed. 
         if self.last_active_window:
             window = self.find_window_by_uuid(self.last_active_window.urn)
-            count = 0
-            while count < 1000 and Gtk.events_pending():
-                count += 1
-                Gtk.main_iteration_do(False)
-                window.show()
-                window.grab_focus()
-                try:
-                    t = GdkX11.x11_get_server_time(window.get_window())
-                except (TypeError, AttributeError):
-                    t = 0
-                window.get_window().focus(t)
-
+            window.present_with_time(t)
         self.prelayout_windows = None
 
     def on_gtk_theme_name_notify(self, settings, prop):
@@ -492,7 +427,9 @@ class Terminator(Borg):
             else:
                 bgcolor = Gdk.RGBA()
                 bgcolor = profiles[profile]['background_color']
-            if profiles[profile]['background_type'] == 'transparent':
+            if profiles[profile]['background_type'] == 'image':
+                backgound_image = profiles[profile]['background_image']
+            if profiles[profile]['background_type'] == 'transparent' or profiles[profile]['background_type'] == 'image':
                 bgalpha = profiles[profile]['background_darkness']
             else:
                 bgalpha = "1"
@@ -539,10 +476,11 @@ class Terminator(Borg):
         css = ""
         if self.config['handle_size'] in range(0, 21):
             css += """
-                .terminator-terminal-window GtkPaned,
-                .terminator-terminal-window paned {
-                    -GtkPaned-handle-size: %s; }
-                """ % self.config['handle_size']
+                .terminator-terminal-window separator {
+                    min-height: %spx;
+                    min-width: %spx; 
+                }
+                """ % (self.config['handle_size'],self.config['handle_size'])
         style_provider = Gtk.CssProvider()
         style_provider.load_from_data(css.encode('utf-8'))
         self.style_providers.append(style_provider)
@@ -581,7 +519,7 @@ class Terminator(Borg):
     def create_group(self, name):
         """Create a new group"""
         if name not in self.groups:
-            dbg('Terminator::create_group: registering group %s' % name)
+            dbg('registering group %s' % name)
             self.groups.append(name)
 
     def closegroupedterms(self, group):
@@ -606,15 +544,14 @@ class Terminator(Borg):
                 if not group in inuse:
                     todestroy.append(group)
 
-            dbg('Terminator::group_hoover: %d groups, hoovering %d' %
+            dbg('%d groups, hoovering %d' %
                     (len(self.groups), len(todestroy)))
             for group in todestroy:
                 self.groups.remove(group)
 
     def group_emit(self, terminal, group, type, event):
         """Emit to each terminal in a group"""
-        dbg('Terminator::group_emit: emitting a keystroke for group %s' %
-                group)
+        dbg('emitting a keystroke for group %s' % group)
         for term in self.terminals:
             if term != terminal and term.group == group:
                 term.vte.emit(type, eventkey2gdkevent(event))
@@ -639,7 +576,17 @@ class Terminator(Borg):
 
         for term in self.get_target_terms(widget):
             idx = terminals.index(term)
-            term.feed(numstr % (idx + 1))
+            term.feed(numstr.encode() % (idx + 1))
+
+    def do_insert_term_name(self, widget):
+        terminals = []
+        for window in self.windows:
+            containers, win_terminals = enumerate_descendants(window)
+            terminals.extend(win_terminals)
+
+        for term in self.get_target_terms(widget):
+            name = term.titlebar.get_custom_string() or term.get_window_title()
+            term.feed(name.encode())
 
     def get_sibling_terms(self, widget):
         termset = []
@@ -660,7 +607,7 @@ class Terminator(Borg):
     def get_focussed_terminal(self):
         """iterate over all the terminals to find which, if any, has focus"""
         for terminal in self.terminals:
-            if terminal.has_focus():
+            if terminal.get_vte().has_focus():
                 return(terminal)
         return(None)
 
@@ -673,14 +620,39 @@ class Terminator(Borg):
     def focus_left(self, widget):
         self.last_focused_term=widget
 
-    def describe_layout(self):
+    def describe_layout(self, save_cwd = False):
         """Describe our current layout"""
         layout = {}
         count = 0
         for window in self.windows:
             parent = ''
-            count = window.describe_layout(count, parent, layout, 0)
+            count = window.describe_layout(count, parent, layout, 0, save_cwd)
 
         return(layout)
 
+    def zoom_in_all(self):
+        for term in self.terminals:
+            term.zoom_in()
+    
+    def zoom_out_all(self):
+        for term in self.terminals:
+            term.zoom_out()
+    
+    def zoom_orig_all(self):
+        for term in self.terminals:
+            term.zoom_orig()
+
+    def new_random_group(self):
+        defaultmembers=[_('Alpha'),_('Beta'),_('Gamma'),_('Delta'),_('Epsilon'),_('Zeta'),_('Eta'),
+                        _('Theta'),_('Iota'),_('Kappa'),_('Lambda'),_('Mu'),_('Nu'),_('Xi'),
+                        _('Omicron'),_('Pi'),_('Rho'),_('Sigma'),_('Tau'),_('Upsilon'),_('Phi'),
+                        _('Chi'),_('Psi'),_('Omega')]
+        currentgroups=set(self.groups)
+        for i in range(1,4):
+            defaultgroups=set(map(''.join, list(itertools.product(defaultmembers,repeat=i))))
+            freegroups = list(defaultgroups-currentgroups)
+            if freegroups:
+                return random.choice(freegroups)
+        return ''
+
 # vim: set expandtab ts=4 sw=4:
