@@ -5,8 +5,6 @@
 from gi.repository import Gtk, Gdk
 from gi.repository import GObject
 from gi.repository import Pango
-import random
-import itertools
 
 from .version import APP_NAME
 from .util import dbg
@@ -104,10 +102,15 @@ class Titlebar(Gtk.EventBox):
     def update(self, other=None):
         """Update our contents"""
         default_bg = False
-        if self.config['title_hide_sizetext']:
-            self.label.set_text("%s" % self.termtext)
-        else:
-            self.label.set_text("%s %s" % (self.termtext, self.sizetext))
+
+        temp_heldtext_str = ''
+        temp_sizetext_str = ''
+
+        if self.terminal.is_held_open:
+            temp_heldtext_str = _('[INACTIVE: Right-Click for Relaunch option] ')
+        if not self.config['title_hide_sizetext']:
+            temp_sizetext_str = " %s" % (self.sizetext)
+        self.label.set_text("%s%s%s" % (temp_heldtext_str, self.termtext, temp_sizetext_str))
 
         if (not self.config['title_use_system_font']) and self.config['title_font']:
             title_font = Pango.FontDescription(self.config['title_font'])
@@ -250,19 +253,7 @@ class Titlebar(Gtk.EventBox):
         if self.terminal.group:
             self.groupentry.set_text(self.terminal.group)
         else:
-            defaultmembers=[_('Alpha'),_('Beta'),_('Gamma'),_('Delta'),_('Epsilon'),_('Zeta'),_('Eta'),
-                            _('Theta'),_('Iota'),_('Kappa'),_('Lambda'),_('Mu'),_('Nu'),_('Xi'),
-                            _('Omicron'),_('Pi'),_('Rho'),_('Sigma'),_('Tau'),_('Upsilon'),_('Phi'),
-                            _('Chi'),_('Psi'),_('Omega')]
-            currentgroups=set(self.terminator.groups)
-            for i in range(1,4):
-                defaultgroups=set(map(''.join, list(itertools.product(defaultmembers,repeat=i))))
-                freegroups = list(defaultgroups-currentgroups)
-                if freegroups:
-                    self.groupentry.set_text(random.choice(freegroups))
-                    break
-            else:
-                self.groupentry.set_text('')
+            self.groupentry.set_text(self.terminator.new_random_group())
         self.groupentry.show()
         self.grouplabel.hide()
         self.groupentry.grab_focus()
@@ -278,7 +269,7 @@ class Titlebar(Gtk.EventBox):
     def groupentry_activate(self, widget):
         """Actually cause a group to be created"""
         groupname = self.groupentry.get_text() or None
-        dbg('Titlebar::groupentry_activate: creating group: %s' % groupname)
+        dbg('creating group: %s' % groupname)
         self.groupentry_cancel(None, None)
         last_focused_term=self.terminator.last_focused_term
         if self.terminal.targets_for_new_group:
